/*
 * storage APIs
 * ------------
 * Mocked version
 * 
 * See: https://developer.mozilla.org/en-US/Add-ons/WebExtensions/Browser_support_for_JavaScript_APIs#storage
 */

const activeListeners = [];

const dispatchChanges = (changes, area) => {
  activeListeners.forEach(listener => listener(changes, area));
};

const createStorageArea = area => ({
  get(keys) {
    const current = JSON.parse(localStorage.getItem(area) || '{}');
    if (typeof keys === 'string') {
      return Promise.resolve(current);
    }
    if (keys instanceof Array) {
      const obj = {};
      keys.forEach(key => {
        obj[key] = current[key];
      });
      return Promise.resolve(obj);
    }
    return Promise.resolve(current);
  },
  set(data) {
    const current = JSON.parse(localStorage.getItem(area) || '{}');
    const updated = Object.assign({}, current, data);
    localStorage.setItem(area, JSON.stringify(updated));
    dispatchChanges(data, area);
    return Promise.resolve();
  },
  remove(keys) {
    const current = JSON.parse(localStorage.getItem(area) || '{}');
    return new Promise(res => {
      if (typeof keys === 'string') {
        delete current[keys];
        localStorage.setItem(area, JSON.stringify(current));
        dispatchChanges({ keys: null }, area);
      } else if (keys instanceof Array) {
        const changes = {};
        keys.forEach(key => {
          changes[key] = null;
          delete current[key];
        });
        localStorage.setItem(area, JSON.stringify(current));
        dispatchChanges(changes, area);
      }
      return Promise.resolve();
    });
  },
  clear() {
    const current = JSON.stringify(localStorage.getItem(area) || '{}');
    Object.keys(current).forEach(key => {
      current[key] = null;
    });
    localStorage.setItem(area, '{}');
    dispatchChanges(current, area);
    return Promise.resolve();
  },
});

const Storage = browser => ({
  // https://developer.mozilla.org/en-US/Add-ons/WebExtensions/API/storage/sync
  sync: createStorageArea('sync'),
  // https://developer.mozilla.org/en-US/Add-ons/WebExtensions/API/storage/local
  local: createStorageArea('local'),
  // https://developer.mozilla.org/en-US/Add-ons/WebExtensions/API/storage/onChanged
  onChanged: {
    addListener(listener) {
      if (!activeListeners.includes(listener)) {
        activeListeners.push(listener);
      }
    },
    removeListener(listener) {
      const index = activeListeners.findIndex(l => l === listener);
      if (index !== null) {
        activeListeners.splice(index, 1);
      }
    },
    hasListener(listener) {
      return activeListeners.some(l => l === listener);
    },
  },
});

export default Storage;
